#ifndef QT_FEATURES_Bluetooth_src_bluetooth_qtbluetooth_config_p_h_H
#define QT_FEATURES_Bluetooth_src_bluetooth_qtbluetooth_config_p_h_H

#define QT_FEATURE_bluez_le 1

#define QT_FEATURE_linux_crypto_api 1

#define QT_FEATURE_winrt_bt -1


#endif // QT_FEATURES_Bluetooth_src_bluetooth_qtbluetooth_config_p_h_H
